/**
 * Assemble full response Structure for clientside polling requests.
 */

var createArrayStore = require('./array-store')
var addProcess = require('./add-process')

var spec = require('./graphs/spectrogram')
var computeSparklines = require('./graphs/sparkline')

module.exports = buildResponse

function buildResponse (processes, hosts, pollInterval, duration) {
  var getApp = createArrayStore({
    hosts: [],
    snapshots: [],
    graphs: {
      rss: [],
      heapTotal: [],
      heapUsed: [],
      spectrogram: [],
      spiffygram: []
    },
    processes: []
  })

  // given the fact that we _may_ have a set of valid proceses
  // loop through all of the process and ensure that they are
  // valid and can be linked to a valid host
  //
  // valid processes mean that they have received both info and process_stats responses
  // valid hosts mean that they have received both system_info and system_stats responses
  //
  // once we have this information we can build the array of applications.  This is done
  // at runtime and is not stored because doing so would add state without any discernible
  // advantage
  var l = processes.length
  for (var i = 0; i < l; i++) {
    var processResponse = processes[i]
    var processId = processResponse.meta.id
    var hostname = processResponse.meta.hostname

    // we need to ensure that the intersection between
    // process_stats & (system_info ∩ system_stats)
    // is valid. if not then we can drop the process, and
    // hopefully pick it up on the next polling cycle.

    if (hosts && hosts[hostname] && hosts[hostname].stats) {
      var host = hosts[processId]

      if (!hosts[hostname].stats.starttime) {
        hosts[hostname].stats.starttime = Math.round((Date.now() - hosts[hostname].stats.uptime * 1000) / 1000) * 1000
      }

      var application = addProcess(processResponse, host, getApp)

      // only insert one instance of the host object per application.
      if (application.hosts.indexOf(hostname) === -1) {
        application.hosts.push(hostname)
      }
    }
  }

  // return the array of _computed_ applications w/ valid (complete) processes
  return getApp.array.map(function (application) {
    computeSparklines(application, hosts, pollInterval, duration)

    spec.computeRSSSpectrogram(application, 25, 10)
    spec.computeCPUSpectrogram(application, 25, 10)
    spec.computeSpiffygram(application, 5, 5)

    return application
  })
}
