/**
 * Wrapper interface for executing N|Solid CLI commands
 */

var split = require('split')
var query = require('nsolid-cli')
var config = require('./config')

module.exports.aggregate = aggregate
module.exports.stream = stream
module.exports.json = json

function json (s) {
  try {
    return JSON.parse(s.replace(/\\/g, '\\\\'))
  } catch (e) {
    return { error: e }
  }
}

function stream (cmd, args) {
  var c = config()
  args.timeout = args.timeout || c.timeout
  return query(cmd, c.hub, args)
}

function aggregate (cmd, args, fn) {
  var s = stream(cmd, args)
  s.on('error', fn)

  var res = []

  s.pipe(split()).on('data', function gotLine (line) {
    var obj = json((line || '').toString())
    if (!obj.error) {
      res.push(obj)
    }
  })

  s.on('end', function streamEnded () {
    fn(null, res)
  })
}
